

proc Set_V1724_Options {} {
    global Options V1724_Base

    load_Options

   if {![info exists Options(V1724_Hist_TraceA.Enable)]}   {set Options(V1724_Hist_TraceA.Enable) 1}
   if {![info exists Options(V1724_Hist_TraceB.Enable)]}   {set Options(V1724_Hist_TraceB.Enable) 0}
   if {![info exists Options(V1724_Hist_Energy.Enable)]}   {set Options(V1724_Hist_Energy.Enable) 1}
   if {![info exists Options(V1724_Hist_TimeH.Enable)]}     {set Options(V1724_Hist_TimeH.Enable) 0}
   if {![info exists Options(V1724_Hist_TimeL.Enable)]}     {set Options(V1724_Hist_TimeL.Enable) 0}
   if {![info exists Options(V1724_Hist_TraceA.channels)]}  {set Options(V1724_Hist_TraceA.channels) 512}
   if {![info exists Options(V1724_Hist_TraceA.offset)]}    {set Options(V1724_Hist_TraceA.offset) 0}
   if {![info exists Options(V1724_Hist_TraceA.shift)]}     {set Options(V1724_Hist_TraceA.shift) 0}
   if {![info exists Options(V1724_Hist_TraceB.channels)]}  {set Options(V1724_Hist_TraceB.channels) 512}
   if {![info exists Options(V1724_Hist_TraceB.offset)]}    {set Options(V1724_Hist_TraceB.offset) 0}
   if {![info exists Options(V1724_Hist_TraceB.shift)]}     {set Options(V1724_Hist_TraceB.shift) 0}
   if {![info exists Options(V1724_Hist_Energy.channels)]} {set Options(V1724_Hist_Energy.channels) 16384}
   if {![info exists Options(V1724_Hist_Energy.offset)]}   {set Options(V1724_Hist_Energy.offset) 0}
   if {![info exists Options(V1724_Hist_Energy.shift)]}    {set Options(V1724_Hist_Energy.shift) 0}
   if {![info exists Options(V1724_Hist_TimeH.channels)]}   {set Options(V1724_Hist_TimeH.channels) 65536}
   if {![info exists Options(V1724_Hist_TimeH.offset)]}     {set Options(V1724_Hist_TimeH.offset) 0}
   if {![info exists Options(V1724_Hist_TimeH.shift)]}      {set Options(V1724_Hist_TimeH.shift) 0}
   if {![info exists Options(V1724_Hist_TimeL.channels)]}   {set Options(V1724_Hist_TimeL.channels) 65536}
   if {![info exists Options(V1724_Hist_TimeL.offset)]}     {set Options(V1724_Hist_TimeL.offset) 0}
   if {![info exists Options(V1724_Hist_TimeL.shift)]}      {set Options(V1724_Hist_TimeL.shift) 0}
   if {![info exists Options(V1724_GroupBase)]}            {set Options(V1724_GroupBase) $V1724_Base}
   set V1724_Base  $Options(V1724_GroupBase)

    update_Options
}


proc Dump_Raw_VME_Data_V1724 {data} {

   set info ""

    foreach word $data {
         if {[string range $word 0 1] != "0x"} then {set word 0x$word}
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         append info "$word "
    }

   append info "\n"

   return $info
}

proc define_V1724_config {} {
    global VMEserver
    global V1724_Base

   set maxmodule [inform vmecontrol inquire v1724count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   if {$maxmodule > 0} then {DefineConfig "1724 0 [format "0x%x" $V1724_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {eg "(register :name '/$VMEserver/v1724#$i.Status'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register v1724#$i.Status"
          clear-footer
          error $m
      }
      DefineConfig "1724 $i [string range [first $m] 2 end]"
   }
}

proc create_V1724_spectra {} {
    global SpectrumServer V1724_Base
    global Options UseGroup

   set maxmodule [inform vmecontrol inquire v1724count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}

   for {set i 1} {$i <= $maxmodule} {incr i} {

     if {$Options(V1724_Hist_Energy.Enable) == 1} {

        for {set j 0} {$j < 8} {incr j} {   

#    histograms for the computed energy

           set spectrumname [SpectrumName v1724#$i.E[set j]]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(V1724_Hist_Energy.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           } else {

               if {$UseGroup == 0} {
                   set group [expr ($i - 1) + $V1724_Base]
                   set item [expr $j + (0 << 3)]
               } else {
                   set group [expr (($i - 1) * 8) + $j + $V1724_Base]
                   set item 0
               }
               map_spectrum  "$spectrumname $group $item $Options(V1724_Hist_Energy.offset) $Options(V1724_Hist_Energy.shift)"

           }
        }
     }

     if {$Options(V1724_Hist_TimeH.Enable) == 1} {

        for {set j 0} {$j < 8} {incr j} {   

#    histograms for the time (MSW)

           set spectrumname [SpectrumName v1724#$i.TH[set j]]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(V1724_Hist_TimeH.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           } else {
               if {$UseGroup == 0} {
                   set group [expr ($i - 1) + $V1724_Base]
                   set item [expr $j + (1 << 3)]
               } else {
                   set group [expr (($i - 1) * 8) + $j + $V1724_Base]
                   set item 1
               }
              map_spectrum  "$spectrumname $group $item $Options(V1724_Hist_TimeH.offset) $Options(V1724_Hist_TimeH.shift)"
           }
        }
     }

    if {$Options(V1724_Hist_TimeL.Enable) == 1} {

        for {set j 0} {$j < 8} {incr j} {   

#    histograms for the time (LSW)

           set spectrumname [SpectrumName v1724#$i.TL[set j]]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(V1724_Hist_TimeL.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           } else {
               if {$UseGroup == 0} {
                   set group [expr ($i - 1) + $V1724_Base]
                   set item [expr $j + (2 << 3)]
               } else {
                   set group [expr (($i - 1) * 8) + $j + $V1724_Base]
                   set item 2
               }
              map_spectrum  "$spectrumname $group $item $Options(V1724_Hist_TimeL.offset) $Options(V1724_Hist_TimeL.shift)"
           }
        }
     }

     if {$Options(V1724_Hist_TraceA.Enable) == 1} {

        for {set j 0} {$j < 8} {incr j} {   

#    histograms for the data trace A

           set spectrumname [SpectrumName v1724#$i.ChA[set j]]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(V1724_Hist_TraceA.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           } else {
              set group  [expr (($i-1) * 8) + $j + 256  + $V1724_Base] 
              map_spectrum  "$spectrumname $group 0 $Options(V1724_Hist_TraceA.offset) $Options(V1724_Hist_TraceA.shift)"
           }
        }
     }

     if {$Options(V1724_Hist_TraceB.Enable) == 1} {

        for {set j 0} {$j < 8} {incr j} {   

#    histograms for the data trace B

           set spectrumname [SpectrumName v1724#$i.ChB[set j]]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(V1724_Hist_TraceB.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           } else {
              set group  [expr (($i-1) * 8) + $j + 384  + $V1724_Base] 
              map_spectrum  "$spectrumname $group 0 $Options(V1724_Hist_TraceB.offset) $Options(V1724_Hist_TraceB.shift)"
           }
        }
     }
   }
}

